import traceback

from IPython.core.display import display, update_display
from IPython.display import HTML
from flown.utils import merge_html

def notebook_api(func):
    def wrapper(*args, restricted: bool = False, display_id: str = None, **kwargs):
        # print(f"args = {args}")
        # print(f"restricted = {restricted}")
        # print(f"display_id = {display_id}")
        # print(f"kwargs = {kwargs}")
        # print(f"func = {func}")

        if restricted:
            display_id = None
        elif display_id is None:
            # print(f"not restricted mode , and display_id is None")
            # print(f"display loading.html ...")
            display_handler = display(HTML(merge_html('_loading.html')), display_id=True)
            display_id = display_handler.display_id

        try:
            content_html_str = func(*args, restricted=restricted, display_id=display_id, **kwargs)
            content_html = HTML(content_html_str)
        except Exception as e:
            st = traceback.format_exc()
            content_html = HTML(f"<pre>{st}</pre>")

        if display_id:
            # print(f'update display...')
            update_display(obj=content_html,
                           display_id=display_id)
        else:
            # print(f'display new...')
            # print(f'html = {content_html_str}')
            display(content_html, display_id=True)

    return wrapper




import mlflow
from mlflow.entities import ViewType
from mlflow.store.tracking.file_store import FileStore
from uuid import uuid4
from operator import attrgetter
from datetime import datetime

class ProjectStore(FileStore):
    def __init__(self, root_directory=None, artifact_root_uri=None):
        """
        Create a new FileStore with the given root directory and a given default artifact root URI.
        """

        # uriから`project://` 部分を削除して使う
        if root_directory.startswith('project://'):
            root_directory = root_directory[10:]
        super().__init__(root_directory, artifact_root_uri)

    def create_experiment(self, name, artifact_location=None):
        self._check_root_dir()
        self._validate_experiment_name(name)
        timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
        experiment_id = f"{timestamp}-{str(uuid4())}"
        return self._create_experiment_with_id(name, experiment_id, artifact_location)


def get_project_store(store_uri, artifact_uri):
    return ProjectStore(root_directory=store_uri, artifact_root_uri=artifact_uri)
